/* AVR MEGA8 */
#include <libavr/device/one_usart.h>

#define SPM_PAGESIZE    64
#define RAMEND          0x045F
#define EEPROMEND       0x01FF
#define FLASHEND        0x1FFF
#define VECTOR_N        18
#define VECTOR_SIZE     2

/* This file is in memory order - to aid checking with the register summary
 * on page 287--288 of the datasheet (revision 2466T-05/08)
 *
 * For registers with named bits, the definitions of those bits follow the
 * register defintion
 */

/* TWI - page 171-174 */
#define TWBR    __IO_MEM(0x20)
#define TWSR    __IO_MEM(0x21)
#define TWPS0   0
#define TWPS1   1
#define TWS3    3
#define TWS4    4
#define TWS5    5
#define TWS6    6
#define TWS7    7
#define TWAR    __IO_MEM(0x22)
#define TWGCE   0
#define TWA0    1
#define TWA1    2
#define TWA2    3
#define TWA3    4
#define TWA4    5
#define TWA5    6
#define TWA6    7
#define TWDR    __IO_MEM(0x23)

/* ADC - page 205-208 */
#define ADCL    __IO_MEM(0x24)
#define ADCH    __IO_MEM(0x25)
#define ADCSRA  __IO_MEM(0x26)
#define ADPS0   0
#define ADPS1   1
#define ADPS2   2
#define ADIE    3
#define ADIF    4
#define ADFR    5
#define ADSC    6
#define ADEN    7
#define ADMUX   __IO_MEM(0x27)
#define MUX0    0
#define MUX1    1
#define MUX2    2
#define MUX3    3
#define ADLAR   5
#define REFS0   6
#define REFS1   7

/* analog comparator - page 194 */
#define ACSR    __IO_MEM(0x28)
#define ACIS0   0
#define ACIS1   1
#define ACIC    2
#define ACIE    3
#define ACI     4
#define ACO     5
#define ACBG    6
#define ACD     7

/* USART - page 153-158 */
#define UBRRL   __IO_MEM(0x29)
#define UCSRB   __IO_MEM(0x2A)
#define TXB8    0
#define RXB8    1
#define UCSZ2   2
#define TXEN    3
#define RXEN    4
#define UDRIE   5
#define TXCIE   6
#define RXCIE   7
#define UCSRA   __IO_MEM(0x2B)
#define MPCM    0
#define U2X     1
#define PE      2
#define DOR     3
#define FE      4
#define UDRE    5
#define TXC     6
#define RXC     7
#define UDR     __IO_MEM(0x2C)

/* SPI - page 129-131 */
#define SPCR    __IO_MEM(0x2D)
#define SPR0    0
#define SPR1    1
#define CPHA    2
#define CPOL    3
#define MSTR    4
#define DORD    5
#define SPE     6
#define SPIE    7
#define SPSR    __IO_MEM(0x2E)
#define SPI2X   0
#define WCOL    6
#define SPIF    7
#define SPDR    __IO_MEM(0x2F)

/* io - page 65 */
#define PIND    __IO_MEM(0x30)
#define DDRD    __IO_MEM(0x31)
#define PORTD   __IO_MEM(0x32)
#define PINC    __IO_MEM(0x33)
#define DDRC    __IO_MEM(0x34)
#define PORTC   __IO_MEM(0x35)
#define PINB    __IO_MEM(0x36)
#define DDRB    __IO_MEM(0x37)
#define PORTB   __IO_MEM(0x38)

/* eeprom - page 20 */
#define EECR    __IO_MEM(0x3C)
#define EERE    0
#define EEWE    1
#define EEMWE   2
#define EERIE   3
#define EEDR    __IO_MEM(0x3D)
#define EEARL   __IO_MEM(0x3E)
#define EEARH   __IO_MEM(0x3F)

/* USART - page 158 */
#define UBRRH   __IO_MEM(0x40)
#define UCSRC   __IO_MEM(0x40)
#define UCPOL   0
#define UCSZ0   1
#define UCSZ1   2
#define USBS    3
#define UPM0    4
#define UPM1    5
#define UMSEL   6
#define URSEL   7

/* watchdog - page 43 */
#define WDTCR   __IO_MEM(0x41)
#define WDP0    0
#define WDP1    1
#define WDP2    2
#define WDE     3
#define WDCE    4

/* timer2 - page 117-119 */
#define ASSR    __IO_MEM(0x42)
#define TCR2UB  0
#define OCR2UB  1
#define TCN2UB  2
#define AS2     3
#define OCR2    __IO_MEM(0x43)
#define TCNT2   __IO_MEM(0x44)
#define TCCR2   __IO_MEM(0x45)
#define CS20    0
#define CS21    1
#define CS22    2
#define WGM21   3
#define COM20   4
#define COM21   5
#define WGM20   6
#define FOC2    7

/* timer1 - page 96-102 */
#define ICR1L   __IO_MEM(0x46)
#define ICR1H   __IO_MEM(0x47)
#define OCR1BL  __IO_MEM(0x48)
#define OCR1BH  __IO_MEM(0x49)
#define OCR1AL  __IO_MEM(0x4A)
#define OCR1AH  __IO_MEM(0x4B)
#define TCNT1L  __IO_MEM(0x4C)
#define TCNT1H  __IO_MEM(0x4D)
#define TCCR1B  __IO_MEM(0x4E)
#define CS10    0
#define CS11    1
#define CS12    2
#define WGM12   3
#define WGM13   4
#define ICES1   6
#define ICNC1   7
#define TCCR1A  __IO_MEM(0x4F)
#define WGM10   0
#define WGM11   1
#define FOC1B   2
#define FOC1A   3
#define COM1B0  4
#define COM1B1  5
#define COM1A0  6
#define COM1A1  7

/* special function io - page 58, 75, 123, 193 */
#define SFIOR   __IO_MEM(0x50)
#define PSR10   0
#define PSR2    1
#define PUD     2
#define ACME    3

/* oscillator calibration - page 31 */
#define OSCCAL  __IO_MEM(0x51)

/* timer0 - page 72 */
#define TCNT0   __IO_MEM(0x52)
#define TCCR0   __IO_MEM(0x53)
#define CS00    0
#define CS01    1
#define CS02    2

/* MCU control / status - page 33, 41, 66 */
#define MCUCSR  __IO_MEM(0x54)
#define PORF    0
#define EXTRF   1
#define BORF    2
#define WDRF    3
#define MCUCR   __IO_MEM(0x55)
#define ISC00   0
#define ISC01   1
#define ISC10   2
#define ISC11   3
#define SM0     4
#define SM1     5
#define SM2     6
#define SE      7

/* TWI - page 171 */
#define TWCR    __IO_MEM(0x56)
#define TWIE    0
#define TWEN    2
#define TWWC    3
#define TWSTO   4
#define TWSTA   5
#define TWEA    6
#define TWINT   7

/* store program memory control - page 213 */
#define SPMCR   __IO_MEM(0x57)
#define SPMEN   0
#define PGERS   1
#define PGWRT   2
#define BLBSET  3
#define RWWSRE  4
#define RWWSB   6
#define SPMIE   7

/* timers - page 73, 102, 122 */
#define TIFR    __IO_MEM(0x58)
#define TOV0    0
#define TOV1    2
#define OCF1B   3
#define OCF1A   4
#define ICF1    5
#define TOV2    6
#define OCF2    7
#define TIMSK   __IO_MEM(0x59)
#define TOIE0   0
#define TOIE1   2
#define OCIE1B  3
#define OCIE1A  4
#define TICIE1  5
#define TOIE2   6
#define OCIE2   7

/* general interrupt flag / control - page 49, 67-68 */
#define GIFR    __IO_MEM(0x5A)
#define INTF0   6
#define INTF1   7
#define GICR    __IO_MEM(0x5B)
#define IVCE    0
#define IVSEL   1
#define INT0    6
#define INT1    7

/* stack pointer / status - page 11-13 */
#define SPL     __IO_MEM(0x5D)
#define SPH     __IO_MEM(0x5E)
#define SREG    __IO_MEM(0x5F)

/* 16bit registers */
#define ADC     __IO_MEM16(0x24)
#define EEAR    __IO_MEM16(0x3E)
#define ICR1    __IO_MEM16(0x46)
#define OCR1B   __IO_MEM16(0x48)
#define OCR1A   __IO_MEM16(0x4A)
#define TCNT1   __IO_MEM16(0x4C)
#define SP      __IO_MEM16(0x5D)

/* interrupts */
#define __vector_INT0           __vector_1
#define __vector_INT1           __vector_2
#define __vector_TIMER2_COMP    __vector_3
#define __vector_TIMER2_OVF     __vector_4
#define __vector_TIMER1_CAPT    __vector_5
#define __vector_TIMER1_COMPA   __vector_6
#define __vector_TIMER1_COMPB   __vector_7
#define __vector_TIMER1_OVF     __vector_8
#define __vector_TIMER0_OVF     __vector_9
#define __vector_SPI_STC        __vector_10
#define __vector_USART_RXC      __vector_11
#define __vector_USART_UDRE     __vector_12
#define __vector_USART_TXC      __vector_13
#define __vector_ADC            __vector_14
#define __vector_EE_RDY         __vector_15
#define __vector_ANA_COMP       __vector_16
#define __vector_TWI            __vector_17
#define __vector_SPM_RDY        __vector_18
