#ifndef __LIBAVR_FLASH__
#define __LIBAVR_FLASH__

#include <stdint.h>
#include <libavr/device.h>

#define FLASH __attribute__((__progmem__))

typedef uint16_t faddr_near_t;
typedef uint32_t faddr_far_t;

#define flash_int8_t const int8_t FLASH
#define flash_int16_t const int16_t FLASH
#define flash_int32_t const int32_t FLASH
#define flash_int64_t const int64_t FLASH
#define flash_uint8_t const uint8_t FLASH
#define flash_uint16_t const uint16_t FLASH
#define flash_uint32_t const uint32_t FLASH
#define flash_uint64_t const uint64_t FLASH

#if defined(FLASH_NEAR) || FLASHEND <= 0xffff

typedef faddr_near_t faddr_t;
#define PTR(x) PTR_NEAR(x)
#define flash_read(address) flash_read_near((faddr_near_t) address)
#define flash_read16(address) flash_read_near16((faddr_near_t) address)
#define flash_read32(address) flash_read_near32((faddr_near_t) address)
#define flash_read64(address) flash_read_near64((faddr_near_t) address)

#else

typedef faddr_far_t faddr_t;
#define PTR(x) PTR_FAR(x)
#define flash_read(address) flash_read_far((faddr_t) address)
#define flash_read16(address) flash_read_far16((faddr_t) address)
#define flash_read32(address) flash_read_far32((faddr_t) address)
#define flash_read64(address) flash_read_far64((faddr_t) address)

#endif

#define PSTR(s) (__extension__({ \
    static char __c[] FLASH = (s); \
    PTR(__c[0]); \
}))

#define PTR_NEAR(x) ((faddr_near_t) &(x))
#define PTR_FAR(x) (__extension__({  \
    faddr_far_t tmp;                 \
    asm ( "ldi    %A0, lo8(%1) \n"   \
          "ldi    %B0, hi8(%1) \n"   \
          "ldi    %C0, hh8(%1) \n"   \
          "clr    %D0          \n"   \
          :                          \
          "=d" (tmp)                 \
          :                          \
          "p"  (&x) );               \
    tmp;                             \
}))

static inline uint8_t flash_read_near(faddr_near_t address) {
    uint8_t data;
    asm ( "lpm  %0, Z"
          :
          "=r" (data)
          :
          "z" (address) );
    return data;
}

static inline uint16_t flash_read_near16(faddr_near_t address) {
    uint16_t data;
    asm ( "lpm  %A0, Z+\n\t"
          "lpm  %B0, Z\n\t"
          :
          "=r" (data), "=z" (address)
          :
          "z" (address) );
    return data;
}

static inline uint32_t flash_read_near32(faddr_near_t address) {
    uint32_t data;
    asm ( "lpm  %A0, Z+\n\t"
          "lpm  %B0, Z+\n\t"
          "lpm  %C0, Z+\n\t"
          "lpm  %D0, Z\n\t"
          :
          "=r" (data), "=z" (address)
          :
          "z" (address) );
    return data;
}

#ifdef GCC_DOESNT_SUPPORT_64BITS_IN_ASM
static inline uint64_t flash_read_near64(faddr_near_t address) {
    uint64_t data;
    asm ( "lpm  %A0, Z+\n\t"
          "lpm  %B0, Z+\n\t"
          "lpm  %C0, Z+\n\t"
          "lpm  %D0, Z+\n\t"
          "lpm  %E0, Z+\n\t"
          "lpm  %F0, Z+\n\t"
          "lpm  %G0, Z+\n\t"
          "lpm  %H0, Z"
          :
          "=r" (data), "=z" (address)
          :
          "z" (address) );
    return data;
}
#endif

#if FLASHEND > 0xffff
static inline uint8_t flash_read_far(faddr_far_t address) {
    uint8_t data;
    asm ( "out %2, %C1\n\t"
          "movw r30, %1\n\t"
          "elpm %0, Z"
          :
          "=r" (data)
          :
          "r" (address), "i" (&RAMPZ - 0x20)
          :
          "r30", "r31");
    return data;
}

static inline uint16_t flash_read_far16(faddr_far_t address) {
    uint16_t data;
    asm ( "out %2, %C1\n\t"
          "movw r30, %1\n\t"
          "elpm %A0, Z+\n\t"
          "elpm %B0, Z\n\t"
          :
          "=r" (data)
          :
          "r" (address), "i" (&RAMPZ - 0x20)
          :
          "r30", "r31");
    return data;
}

static inline uint32_t flash_read_far32(faddr_far_t address) {
    uint32_t data;
    asm ( "out %2, %C1\n\t"
          "movw r30, %1\n\t"
          "elpm %A0, Z=\n\t"
          "elpm %B0, Z+\n\t"
          "elpm %C0, Z+\n\t"
          "elpm %D0, Z\n\t"
          :
          "=r" (data)
          :
          "r" (address), "i" (&RAMPZ - 0x20)
          :
          "r30", "r31");
    return data;
}

#ifdef GCC_DOESNT_SUPPORT_64BITS_IN_ASM
static inline uint64_t flash_read_far64(faddr_far_t address) {
    uint64_t data;
    asm ( "out %2, %C1\n\t"
          "movw r30, %1\n\t"
          "elpm %A0, Z=\n\t"
          "elpm %B0, Z+\n\t"
          "elpm %C0, Z+\n\t"
          "elpm %D0, Z+\n\t"
          "elpm %E0, Z+\n\t"
          "elpm %F0, Z+\n\t"
          "elpm %G0, Z+\n\t"
          "elpm %H0, Z\n\t"
          :
          "=r" (data)
          :
          "r" (address), "i" (&RAMPZ - 0x20)
          :
          "r30", "r31");
    return data;
}
#endif
#endif

#endif
